@@ -103,18 +103,88 @@ class ConfigManager:
     def isTroop(self, name):
         return name in self.game_config['types']['tokens']['troop_tokens']
 
+    def isTroopSpecial(self, name):
+        if not self.isTroop(name):
+            raise ValueError('{0} is not a troop token at all'.format(name))
+        if name in ['spiritual_advisor', 'fedaykin', 'sardaukar']:
+            return True
+        return False
+
+    def getGamePhases(self):
+        return ['setup', 'storm', 'spice_blow', 'nexus', 'choam_charity', 'bidding', 'revival', 'shipment_and_movement', 'battle', 'spice_harvest', 'mentat_pause']
+
+    def getGamePhasesChoices(self):
+        phases = self.getGamePhases()
+        choices = []
+        for phase in phases:
+            choices.append({
+                'name': (phase+'_phase').replace('_', ' ').title(),
+                'value': phase
+            })
+        return choices
+
+    def isGamePhaseValid(self, phase):
+        return phase in self.getGamePhases()
+
     def getRadius(self, element):
         diameter = None
         if self.isLeader(element):
             diameter = self.game_config['dimensions']['leader']
         elif self.isTroop(element):
             diameter = self.game_config['dimensions']['troop']
+        elif element == 'worm':
+            diameter = self.game_config['dimensions']['leader']
         radius = diameter/2
         return radius
 
     def getAreas(self):
         return self.all_areas
 
+    def getTroopTypes(self, faction):
+        troop_types = [faction + '_troops']
+        if faction == 'bene_gesserit':
+            troop_types.append('spiritual_advisor')
+        if faction == 'fremen':
+            troop_types.append('fedaykin')
+        if faction == 'emperor':
+            troop_types.append('sardaukar')
+        return troop_types
+
+    def getTroopTypesAll(self):
+        troop_types = [faction + '_troops' for faction in self.getFactions()]
+        troop_types.append('spiritual_advisor')
+        troop_types.append('fedaykin')
+        troop_types.append('sardaukar')
+        return troop_types
+
+    def getTroopTypesChoices(self, sort=False, reverse=False, swap=False):
+        troop_types = self.getTroopTypesAll()
+        choices = []
+        for troop_type in troop_types:
+            troop_type_value = troop_type.replace('_', ' ').title()
+            name, value = troop_type, troop_type_value
+            if swap:
+                value, name = troop_type, troop_type_value
+            choices.append({
+                'name': name,
+                'value': value
+            })
+        if sort:
+            options = sorted(choices, key=lambda choice: len(choice['name']), reverse=True)
+        return choices
+
+    def getFactionFromTroopType(self, troop_type):
+        for faction in self.getFactions():
+            if troop_type == faction + '_troops':
+                return faction
+        if troop_type == 'spiritual_advisor':
+            return 'bene_gesserit'
+        if troop_type == 'fedaykin':
+            return 'fremen'
+        if troop_type == 'sardaukar':
+            return 'emperor'
+        return None
+
     def getRegions(self):
         return self.game_config['generated']['regions']
 
@@ -122,7 +192,7 @@ class ConfigManager:
         regions = self.getRegions()
         choices = []
         for region in regions:
-            region_value = region.replace('R', 'region ')
+            region_value = region.replace('R', 'Sector ')
             name, value = region, region_value
             if swap:
                 value, name = region, region_value
@@ -134,6 +204,18 @@ class ConfigManager:
             options = sorted(choices, key=lambda choice: len(choice['name']), reverse=True)
         return choices
 
+    def getLocationsOfSectors(self):
+        data = {}
+        for location, sectors in self.game_config['generated']['location_regions'].items():
+            if location == 'arrakis':
+                continue
+            for sector in sectors:
+                if sector not in data.keys():
+                    data[sector] = []
+                data[sector].append(location)
+                data[sector] = list(set(data[sector]))
+        return data
+
     def getLocations(self):
         return self.game_config['generated']['locations']
 
@@ -219,7 +301,24 @@ class ConfigManager:
                 'value': value
             })
         if sort:
-            choices = sorted(choices, reverse=reverse)
+            choices = sorted(choices, key=lambda choice: choice['name'], reverse=reverse)
+        return choices
+
+
+    def getSpiceAreasChoices(self, sort=False, reverse=False, swap=False):
+        choices = []
+        for area in self.game_config['types']['areas']['spice']:
+            area_value = area.replace('_spice', '')
+            area_name = area_value.replace('_', ' ').title()
+            name, value = area_name, area_value
+            if swap:
+                value, name = area_name, area_value
+            choices.append({
+                'name': name,
+                'value': value
+            })
+        if sort:
+            choices = sorted(choices, key=lambda choice: choice['name'], reverse=reverse)
         return choices
 
 
@@ -309,7 +408,7 @@ class RenderingProcessor:
         problem = TokenPlacementProblem(polygons_maximize_overlap, avoid_overlap_areas, target_radius, tolerance=0.01)
         result = genetic(problem, population_size=75, mutation_chance=0.15, iterations_limit=120)
         x, y = result.state
-        token_type = None
+        token_type = 'leader_like'
         if self.manager.isLeader(element_name):
             token_type = 'leader'
         if self.manager.isTroop(element_name):
@@ -332,7 +431,7 @@ class RenderingProcessor:
         for i, (name, amount) in enumerate(zip(names, amounts)):
             x = result.state[i*2]
             y = result.state[i*2+1]
-            token_type = None
+            token_type = 'leader_like'
             # print('multiplace')
             # print(name)
             if self.manager.isLeader(name):
@@ -416,14 +515,20 @@ class RenderingProcessor:
         for wheel in wheel_leaders:
             if wheel in game_state['areas'].keys():
                 leader = game_state['areas'][wheel]
-                file = self.manager.getFile(leader)
-                game_state['visual'][wheel] = file
+                if leader is not None:
+                    file = self.manager.getFile(leader)
+                    game_state['visual'][wheel] = file
         wheel_players = ['wheel_attacker_player', 'wheel_defender_player']
         wheel_players_refs = {}
         for wheel in wheel_players:
             if wheel in game_state['areas'].keys():
-                player_key = game_state['areas'][wheel]
-                faction_key = game_state['meta']['factions'][player_key]
+                faction_key = game_state['areas'][wheel]
+                player_key = None
+                for f, key in game_state['meta']['factions'].items():
+                    if key == faction_key:
+                        player_key = f
+                if player_key is None:
+                    continue
                 faction_name = self.manager.getFactionName(faction_key)
                 player_name = game_state['meta']['usernames'][player_key]
                 game_state['visual'][wheel + '_faction'] = faction_name
@@ -484,7 +589,7 @@ class RenderingProcessor:
                         removal = area, region, token
                     elif token not in game_state['areas'][area][region].keys():
                         removal = area, region, token
-                    print(removal)
+                    # print(removal)
                     if removal is not None:
                         to_remove.append(removal)
         # remove gathered
@@ -523,6 +628,45 @@ class OriginatorTruthsayer(OriginatorJSON):
         if target_region[0] != 'R' and target_region != 'whole':
             raise ValueError('Invalid region')
 
+    def getAreasOfPresence(self, faction):
+        troop_types = self.processor.manager.getTroopTypes(faction)
+        areas = {}
+        for area, content in self._object_state['areas'].items():
+            if type(content) is not dict:
+                continue
+            for region in content.keys():
+                for troop_type in troop_types:
+                    if troop_type in content[region].keys():
+                        if content[region][troop_type] > 0:
+                            if area not in areas.keys():
+                                areas[area] = {}
+                            if region not in areas[area].keys():
+                                areas[area][region] = {}
+                            areas[area][region][troop_type] = content[region][troop_type]
+        return areas
+
+    def getAvailableReserves(self, faction):
+        if faction not in self._object_state['hidden']['reserves'].keys():
+            raise ValueError('Incorrect faction')
+        return self._object_state['hidden']['reserves'][faction]
+
+    def getAvailableCards(self, faction):
+        if faction not in self._object_state['hidden']['cards'].keys():
+            raise ValueError('Incorrect faction')
+        return self._object_state['hidden']['cards'][faction]
+
+    def getSelectedCards(self, faction):
+        participants = [
+            self._object_state['areas']['wheel_attacker_player'],
+            self._object_state['areas']['wheel_defender_player']
+        ]
+        if faction not in participants:
+            raise ValueError('Player is not a battle participant')
+        key = 'wheel_attacker_cards'
+        if faction == participants[1]:
+            key = 'wheel_defender_cards'
+        return self._object_state['areas'][key]
+
     def move(self, faction, source_area, source_region, target_area, target_region, N, troop_type=None):
         self.validateMapLocation(source_area, source_region)
         self.validateMapLocation(target_area, target_region)
@@ -568,8 +712,85 @@ class OriginatorTruthsayer(OriginatorJSON):
         cmd = '/{0} {1} {2} {3} {4}'.format('ship', faction, target_area, target_region, str(N))
         self.appendCMD(cmd)
 
-    def change(self, target_area, target_region, N):
-        pass
+    def placeLeader(self, faction, target_area, target_region, leader):
+        self.validateMapLocation(target_area, target_region)
+        if target_area not in self._object_state['areas'].keys():
+            self._object_state['areas'][target_area] = {}
+        if target_region not in self._object_state['areas'][target_area].keys():
+            self._object_state['areas'][target_area][target_region] = {}
+        if leader not in self._object_state['areas'][target_area][target_region].keys():
+            self._object_state['areas'][target_area][target_region][leader] = 1
+        else:
+            # take leader back to reserves
+            self._object_state['areas'][target_area][target_region][leader] = 0
+            self._object_state['hidden']['leaders'][faction].append(leader)
+            cmd = '/{0} {1} {2} {3} {4}'.format('placeLeader', faction, target_area, target_region, leader)
+            self.appendCMD(cmd)
+            return None
+        found = False
+        if leader in self._object_state['hidden']['leaders'][faction]:
+            found = True
+            self._object_state['hidden']['leaders'][faction].remove(leader)
+        elif leader == self._object_state['areas']['wheel_attacker_leader']:
+            found = True
+            self._object_state['areas']['wheel_attacker_leader'] = None
+        elif leader == self._object_state['areas']['wheel_defender_leader']:
+            found = True
+            self._object_state['areas']['wheel_defender_leader'] = None
+        if not found:
+            raise ValueError('Leader not found in available resources')
+        cmd = '/{0} {1} {2} {3} {4}'.format('placeLeader', faction, target_area, target_region, leader)
+        self.appendCMD(cmd)
+
+    def worm(self, target_area, target_region):
+        self.validateMapLocation(target_area, target_region)
+        if target_area not in self._object_state['areas'].keys():
+            self._object_state['areas'][target_area] = {}
+        if target_region not in self._object_state['areas'][target_area].keys():
+            self._object_state['areas'][target_area][target_region] = {}
+        token = 'worm'
+        if token not in self._object_state['areas'][target_area][target_region].keys():
+            self._object_state['areas'][target_area][target_region][token] = 1
+        else:
+            del self._object_state['areas'][target_area][target_region][token]
+        cmd = '/{0} {1} {2}'.format('worm', target_area, target_region)
+        self.appendCMD(cmd)
+
+    def turn(self, N):
+        if N < 1:
+            raise ValueError('Turn has to be greater or equal than 1')
+        self._object_state['meta']['texts']['game_turn'] = N
+        cmd = '/{0} {1}'.format('turn', str(N))
+        self.appendCMD(cmd)
+
+    def phase(self, name):
+        if not self.processor.manager.isGamePhaseValid(name):
+            raise ValueError('Invalid game phase')
+        self._object_state['meta']['texts']['game_phase'] = name
+        cmd = '/{0} {1}'.format('phase', name)
+        self.appendCMD(cmd)
+
+    def change(self, faction, target_area, target_region):
+        if faction != 'bene_gesserit':
+            raise ValueError('Only Bene Gesserit can change unit types')
+        if target_area not in self._object_state['areas'].keys():
+            raise ValueError('Not enough troops')
+        if target_region not in self._object_state['areas'][target_area].keys():
+            raise ValueError('Not enough troops')
+        available = self._object_state['areas'][target_area][target_region].keys()
+        if 'bene_gesserit_troops' in available:
+            source_troop_type = 'bene_gesserit_troops'
+            target_troop_type = 'spiritual_advisor'
+        elif 'bene_gesserit_troops' in available:
+            target_troop_type = 'bene_gesserit_troops'
+            source_troop_type = 'spiritual_advisor'
+        else:
+            raise ValueError('Not enough troops')
+        n = self._object_state['areas'][target_area][target_region][source_troop_type]
+        del self._object_state['areas'][target_area][target_region][source_troop_type]
+        self._object_state['areas'][target_area][target_region][target_troop_type] = n
+        cmd = '/{0} {1} {2}'.format('change', target_area, target_region)
+        self.appendCMD(cmd)
 
     # payee is the one receiving
     def pay(self, spice, payor, payee):
@@ -604,7 +825,7 @@ class OriginatorTruthsayer(OriginatorJSON):
         cmd = '/{0} {1} {2} {3}'.format('harvest', faction, target_area_spice, str(N))
         self.appendCMD(cmd)
 
-    def kill(self, leader):
+    def killLeader(self, leader):
         if not self.processor.manager.isLeader(leader):
             raise ValueError('Invalid leader')
         faction = self.processor.manager.getLeadersFaction(leader)
@@ -619,6 +840,29 @@ class OriginatorTruthsayer(OriginatorJSON):
         cmd = '/{0} {1}'.format('kill', leader)
         self.appendCMD(cmd)
 
+    def kill(self, faction, source_territory, source_region, n, troop_type):
+        if source_territory not in self._object_state['areas'].keys():
+            raise ValueError('Not enough troops')
+        if source_region not in self._object_state['areas'][source_territory].keys():
+            raise ValueError('Not enough troops')
+        available = self._object_state['areas'][source_territory][source_region].keys()
+        if troop_type not in available:
+            raise ValueError('Not enough troops')
+        if self._object_state['areas'][source_territory][source_region][troop_type] < n:
+            raise ValueError('Not enough troops')
+        self._object_state['areas'][source_territory][source_region][troop_type] -= n
+        if 'tleilaxu_tanks' not in self._object_state['areas'].keys():
+            self._object_state['areas']['tleilaxu_tanks'] = {
+                'whole': {}
+            }
+        if 'whole' not in self._object_state['areas']['tleilaxu_tanks'].keys():
+            self._object_state['areas']['tleilaxu_tanks'] = {
+                'whole': {}
+            }
+        if troop_type not in self._object_state['areas']['tleilaxu_tanks']['whole'].keys():
+            self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type] = 0
+        self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type] += n
+
     def reviveLeader(self, leader):
         if not self.processor.manager.isLeader(leader):
             raise ValueError('Invalid leader')
@@ -634,96 +878,153 @@ class OriginatorTruthsayer(OriginatorJSON):
         cmd = '/{0} {1}'.format('reviveLeader', leader)
         self.appendCMD(cmd)
 
-    def revive(self, faction, n, special=False):
-        pass
+    def revive(self, caller_faction, n, troop_type):
+        owner_faction = self.processor.manager.getFactionFromTroopType(troop_type)
+        if owner_faction is None:
+            raise ValueError('Unrecognized troop type: ' + troop_type)
+        if 'tleilaxu_tanks' not in self._object_state['areas'].keys():
+            raise ValueError('Troops not in Tleilaxu Tanks')
+        if 'whole' not in self._object_state['areas']['tleilaxu_tanks'].keys():
+            raise ValueError('Troops not in Tleilaxu Tanks')
+        if troop_type not in self._object_state['areas']['tleilaxu_tanks']['whole'].keys():
+            raise ValueError('Troops not in Tleilaxu Tanks')
+        if self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type] < n:
+            raise ValueError('Not enough troops in Tleilaxu Tanks')
+        self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type] -= n
+        if self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type] == 0:
+            del self._object_state['areas']['tleilaxu_tanks']['whole'][troop_type]
+        self._object_state['hidden']['reserves'][owner_faction][troop_type] += n
+        cmd = '/{0} {1} {2}'.format('revive', str(n), troop_type)
+        self.appendCMD(cmd)
 
-    def lead(self, player, leader):
+    def lead(self, faction, leader):
         participants = [
             self._object_state['areas']['wheel_attacker_player'],
             self._object_state['areas']['wheel_defender_player']
         ]
-        if player not in participants:
+        if faction not in participants:
             raise ValueError('Player is not a battle participant')
         if not self.processor.manager.isLeader(leader):
             raise ValueError('Invalid leader')
-        if player == participants[0]:
+        if faction == participants[0]:
             self._object_state['areas']['wheel_attacker_leader'] = leader
-        if player == participants[1]:
+        if faction == participants[1]:
             self._object_state['areas']['wheel_defender_leader'] = leader
 
-    def treachery(self, player, card):
+    def treachery(self, faction, card_type, card):
+        print('treachery', card_type, card)
         participants = [
             self._object_state['areas']['wheel_attacker_player'],
             self._object_state['areas']['wheel_defender_player']
         ]
-        if player not in participants:
+        if faction not in participants:
             raise ValueError('Player is not a battle participant')
-        faction = self._object_state['meta']['factions'][player]
-        if card not in self._object_state['hidden']['cards'][faction]:
+        if card_type not in self._object_state['hidden']['cards'][faction].keys():
             raise ValueError('Player does not have that card in the hand')
-        if card in self._object_state['hidden']['cards'][faction]: self._object_state['hidden']['cards'][faction].remove(card)
-        if player == participants[0]:
-            self._object_state['areas']['wheel_attacker_cards'] += [card]
-        if player == participants[1]:
-            self._object_state['areas']['wheel_defender_cards'] += [card]
+        if card not in self._object_state['hidden']['cards'][faction][card_type]:
+            raise ValueError('Player does not have that card in the hand')
+        if card in self._object_state['hidden']['cards'][faction][card_type]:
+            self._object_state['hidden']['cards'][faction][card_type].remove(card)
+        if faction == participants[0]:
+            if card_type not in self._object_state['areas']['wheel_attacker_cards'].keys():
+                self._object_state['areas']['wheel_attacker_cards'][card_type] = []
+            self._object_state['areas']['wheel_attacker_cards'][card_type] += [card]
+        if faction == participants[1]:
+            if card_type not in self._object_state['areas']['wheel_defender_cards'].keys():
+                self._object_state['areas']['wheel_defender_cards'][card_type] = []
+            self._object_state['areas']['wheel_defender_cards'][card_type] += [card]
 
-    def discard(self, player, card):
+    def reverseTreachery(self, faction, card_type, card):
+        print('rev-treachery', card_type, card)
         participants = [
             self._object_state['areas']['wheel_attacker_player'],
             self._object_state['areas']['wheel_defender_player']
         ]
-        if player not in participants:
+        if faction not in participants:
             raise ValueError('Player is not a battle participant')
-        faction = self._object_state['meta']['factions'][player]
         key = 'wheel_attacker_cards'
-        if player == participants[1]:
+        if faction == participants[1]:
             key = 'wheel_defender_cards'
-        if card not in self._object_state['areas'][key]:
+        if card not in self._object_state['areas'][key][card_type]:
             raise ValueError('This card is not part of the battle plan')
-        self._object_state['areas'][key].remove(card)
-        self._object_state['hidden']['discarded'].append(card)
-        cmd = '/{0} {1} {2}'.format('discard', player, card)
-        self.appendCMD(cmd)
+        self._object_state['areas'][key][card_type].remove(card)
+        self._object_state['hidden']['cards'][faction][card_type].append(card)
 
-    def takeback(self, player):
+    def discard(self, faction, card_type, card):
         participants = [
             self._object_state['areas']['wheel_attacker_player'],
             self._object_state['areas']['wheel_defender_player']
         ]
-        if player not in participants:
+        if faction not in participants:
             raise ValueError('Player is not a battle participant')
-        faction = self._object_state['meta']['factions'][player]
         key = 'wheel_attacker_cards'
-        if player == participants[1]:
+        if faction == participants[1]:
             key = 'wheel_defender_cards'
-        for card in self._object_state['areas'][key]:
-            self._object_state['hidden']['cards'][faction].append(card)
-        self._object_state['areas'][key] = []
-        cmd = '/{0} {1}'.format('takeback', player)
+        if card not in self._object_state['areas'][key][card_type]:
+            raise ValueError('This card is not part of the battle plan')
+        self._object_state['areas'][key][card_type].remove(card)
+        if card_type not in self._object_state['hidden']['discarded'].keys():
+            self._object_state['hidden']['discarded'][card_type] = []
+        self._object_state['hidden']['discarded'][card_type].append(card)
+        cmd = '/{0} {1} {2}'.format('discard', faction, card)
         self.appendCMD(cmd)
 
-    def battle(self, aggressor_player, defender_player):
-        self._object_state['areas']['wheel_attacker_player'] = aggressor_player
-        self._object_state['areas']['wheel_defender_player'] = defender_player
-        self._object_state['areas']['wheel_attacker_cards'] = []
-        self._object_state['areas']['wheel_defender_cards'] = []
+    def discardHand(self, faction, card_type, card):
+        if faction not in self._object_state['hidden']['reserves'].keys():
+            raise ValueError('Incorrect faction')
+        if card_type not in self._object_state['hidden']['cards'][faction].keys():
+            raise ValueError('Player does not have that card in the hand')
+        if card in self._object_state['hidden']['cards'][faction][card_type]: self._object_state['hidden']['cards'][faction][card_type].remove(card)
+        if card_type not in self._object_state['hidden']['discarded'].keys():
+            self._object_state['hidden']['discarded'][card_type] = []
+        self._object_state['hidden']['discarded'][card_type].append(card)
+        cmd = '/{0} {1} {2}'.format('discard', faction, card)
+        self.appendCMD(cmd)
+
+    def takeback(self, faction):
+        participants = [
+            self._object_state['areas']['wheel_attacker_player'],
+            self._object_state['areas']['wheel_defender_player']
+        ]
+        if faction not in participants:
+            raise ValueError('Player is not a battle participant')
+        key = 'wheel_attacker_cards'
+        if faction == participants[1]:
+            key = 'wheel_defender_cards'
+        for card_type, card_list in self._object_state['areas'][key].items():
+            if card_type not in self._object_state['hidden']['cards'][faction].keys():
+                self._object_state['hidden']['cards'][faction][card_type] = []
+            for card in card_list:
+                self._object_state['hidden']['cards'][faction][card_type].append(card)
+        self._object_state['areas'][key] = []
+        cmd = '/{0} {1}'.format('takeback', faction)
+        self.appendCMD(cmd)
+
+    def traitor(self, caller_faction, leader):
+        # check if caller has a traitor card for this leader
+        if 'traitor_' + leader not in self._object_state['hidden']['cards'][caller_faction]:
+            raise ValueError('Player does not have this traitor')
+        return True
+
+    def battle(self, aggressor_faction, defender_faction):
+        self._object_state['areas']['wheel_attacker_player'] = aggressor_faction
+        self._object_state['areas']['wheel_defender_player'] = defender_faction
+        self._object_state['areas']['wheel_attacker_cards'] = {}
+        self._object_state['areas']['wheel_defender_cards'] = {}
         self._object_state['areas']['wheel_attacker_value'] = 0
         self._object_state['areas']['wheel_defender_value'] = 0
         self._object_state['areas']['wheel_attacker_leader'] = None
         self._object_state['areas']['wheel_defender_leader'] = None
-        cmd = '/{0} {1} {2}'.format('battle', aggressor_player, defender_player)
+        cmd = '/{0} {1} {2}'.format('battle', aggressor_faction, defender_faction)
         self.appendCMD(cmd)
 
-    def deployment(self, player, N):
+    def deployment(self, faction, N):
         participants = [
             self._object_state['areas']['wheel_attacker_player'],
             self._object_state['areas']['wheel_defender_player']
         ]
-        if player not in participants:
-            raise ValueError('Player is not a battle participant')
-        faction = self._object_state['meta']['factions'][player]
         key = 'wheel_attacker_value'
-        if player == participants[1]:
+        if faction == participants[1]:
             key = 'wheel_defender_value'
         self._object_state['areas'][key] = N
 
@@ -750,14 +1051,14 @@ class OriginatorTruthsayer(OriginatorJSON):
         return game_id, card
 
     def draw(self, faction, deck):
-        for name in self._object_state['hidden']['decks'].keys():
-            print(name)
         if deck not in self._object_state['hidden']['decks'].keys():
             raise ValueError('Invalid deck name')
         cmd = '/{0} {1} {2}'.format('draw', faction, deck)
         self.appendCMD(cmd)
         card_id = self._object_state['hidden']['decks'][deck].pop()
-        self._object_state['hidden']['cards'][faction].append(card_id)
+        if deck not in self._object_state['hidden']['cards'][faction].keys():
+            self._object_state['hidden']['cards'][faction][deck] = []
+        self._object_state['hidden']['cards'][faction][deck].append(card_id)
         card = self.cards_manager.card_objects[card_id]
         game_id = self._object_state['meta']['texts']['game_id']
         return game_id, card
@@ -836,6 +1137,10 @@ class OriginatorTruthsayer(OriginatorJSON):
             del self._object_state['meta']['texts']['commands'][0]
         # print(self._object_state['meta']['texts'])
 
+    def shieldWallDestroyed(self, state: bool):
+        self._object_state['meta']['shield_wall_destroyed'] = state
+        self.appendCMD('/shield_wall {0}'.format(str(state)))
+
     def initgame(self):
         deck_treachery = list(self.cards_manager.headers_deck_treachery)
         deck_spice = list(self.cards_manager.headers_deck_spice)
@@ -858,7 +1163,7 @@ class OriginatorTruthsayer(OriginatorJSON):
             'storm': random.randint(1, 18)
         }
         for player, faction in meta['factions'].items():
-            hidden['cards'][faction] = []
+            hidden['cards'][faction] = {}
             hidden['leaders'][faction] = self.processor.manager.getLeaders(faction)
             if faction == 'atreides':
                 hidden['reserves']['atreides'] = {
@@ -944,7 +1249,7 @@ class OriginatorTruthsayer(OriginatorJSON):
                 'defender': None
             },
             'cards': {},
-            'discarded': [],
+            'discarded': {},
             'height': 0,
             'decks': {}
         }
