@@ -82,7 +82,9 @@ class Renderer:
         self.fnt_troop = ImageFont.truetype(filename, 22)
         del filename
 
-    def renderTroop(self, x, y, faction, number):
+    # for text use self.d making sure text is on top of all the tokens
+    # for drawing tokens use d that would ensure they are under the spiceglow
+    def renderTroop(self, d, x, y, faction, number):
         colors = {
             'spiritual_advisor': '#274587',
             'bene_gesserit_troops': '#274587',
@@ -100,7 +102,7 @@ class Renderer:
             y-self.troop_size/2,
             x+self.troop_size/2,
             y+self.troop_size/2)
-        self.d.ellipse(box, fill=fill, outline ='black', width=2)
+        d.ellipse(box, fill=fill, outline ='black', width=2)
         text_fill = 'white'
         if faction in ['spiritual_advisor', 'sardaukar', 'fedaykin']:
             text_fill = 'black'
@@ -109,12 +111,16 @@ class Renderer:
                 y-self.troop_size/2+self.troop_edge,
                 x+self.troop_size/2-self.troop_edge,
                 y+self.troop_size/2-self.troop_edge)
-            self.d.ellipse(box, fill='white', outline ='black', width=2)
+            d.ellipse(box, fill='white', outline ='black', width=2)
         text = str(number)
         w, h = self.fnt_troop.getsize(text)
         self.d.text((x, y+h/2-2), text, font=self.fnt_troop, fill=text_fill, anchor='ms')
 
     def renderTroops(self):
+        # troop token layer
+        layer = Image.new('RGBA', self.canvas.size, (255,255,255,0))
+        d = ImageDraw.Draw(layer)
+        # handle the generation
         areas = self.game_config['generated']['areas']['polygons']
         type_point = self.game_config['types']['areas']['point']
         for area_name, region_object in self.game_state['visual'].items():
@@ -133,6 +139,9 @@ class Renderer:
                 # print(token_object)
                 if region_name[0] != 'R' and region_name != 'whole':
                     continue
+                # in case if there are any leaders there
+                self.renderLeaders(token_object)
+                # troop tokens
                 for token_name, token_instance in token_object.items():
                     # print('token instnace')
                     # print(token_instance)
@@ -142,7 +151,11 @@ class Renderer:
                     if token_type == 'troop_token':
                         faction = token_instance['token']
                         number = token_instance['c']
-                        self.renderTroop(x, y, faction, number)
+                        if number == 0:
+                            continue
+                        self.renderTroop(d, x, y, faction, number)
+        # compose the tokens layer
+        self.canvas = Image.alpha_composite(self.canvas, layer)
 
     def calculateFactionLeadersPositions(self):
         positions = []
@@ -252,31 +265,33 @@ class Renderer:
             text = self.texts['player_names'][area_name] + '\n' + self.texts['user_ids'][area_name]
             self.d.text((x_info, y_info), text, font=self.fnt, fill='white')
 
+    def renderLeaders(self, map_object):
+        # print(map_object)
+        for leader, token_instance in map_object.items():
+            token_type = token_instance['type']
+            if token_type not in ['leader', 'leader_like']:
+                continue
+            disc_filename = self.game_config['files'][leader]
+            x = int(token_instance['x'])
+            y = int(token_instance['y'])
+            filename = pkg_resources.open_binary(assets, disc_filename)
+            token = Image.open(filename)
+            token = token.convert('RGBA')
+            token = token.resize((self.leader_size, self.leader_size), Image.ANTIALIAS)
+            width_token, height_token = token.size
+            dx = int(self.leader_size/2)
+            dy = int(self.leader_size/2)
+            box_target = (x-dx, y-dy, x+dx, y+dy)
+            self.canvas.paste(token, box_target, mask=token)
+            break
+
     def renderTleilaxuTanks(self):
         if 'tleilaxu_tanks' not in self.game_state['visual'].keys():
             return None
         if 'whole' not in self.game_state['visual']['tleilaxu_tanks'].keys():
             return None
-        values = self.deck_generator['traitor_deck']['values']
-        for leader, token_instance in self.game_state['visual']['tleilaxu_tanks']['whole'].items():
-            for entry in values:
-                token_name = entry[0]
-                if leader != token_name:
-                    continue
-                leader_faction = entry[2]
-                disc_filename = leader_faction + '_' + leader + '.png'
-                x = int(token_instance['x'])
-                y = int(token_instance['y'])
-                filename = pkg_resources.open_binary(assets, disc_filename)
-                token = Image.open(filename)
-                token = token.convert('RGBA')
-                token = token.resize((self.leader_size, self.leader_size), Image.ANTIALIAS)
-                width_token, height_token = token.size
-                dx = int(self.leader_size/2)
-                dy = int(self.leader_size/2)
-                box_target = (x-dx, y-dy, x+dx, y+dy)
-                self.canvas.paste(token, box_target, mask=token)
-                break
+        map_object = self.game_state['visual']['tleilaxu_tanks']['whole']
+        self.renderLeaders(map_object)
 
     def renderGameInfo(self):
         # game info
@@ -285,7 +300,9 @@ class Renderer:
         dx = int(self.leader_r*math.cos(angle))
         x_info = int(self.width_canvas/2)+dx-int(self.leader_size/2)
         y_info = 110
-        game_info = 'Game {0}\nTurn {1}\n{2} phase'.format(self.texts['game_id'], str(self.texts['game_turn']), self.texts['game_phase'])
+        game_phase_name = (self.texts['game_phase']+'_phase').replace('_', ' ').title()
+        game_name = self.texts['game_name'].replace('-', ' ').title()
+        game_info = '{0}\nTurn {1}\n{2}'.format(game_name, str(self.texts['game_turn']), game_phase_name)
         text = game_info
         self.canvas, w, h = self.renderText(text, self.fnt, 'white', x_info, y_info, anchor=None)
 
@@ -299,7 +316,26 @@ class Renderer:
         y_info = int(self.height_canvas-lh-20)
         self.canvas, _, _ = self.renderText(text, self.fnt, 'white', x_info, y_info, anchor=None)
 
+    def shieldWall(self):
+        if not self.game_state['meta'].get('shield_wall_destroyed', False):
+            return None
+        filename = self.game_config['files']['shield_wall_destroyed']
+        filename = pkg_resources.open_binary(assets, filename)
+        token = Image.open(filename)
+        token = token.convert('RGBA')
+        width_token, height_token = token.size
+        del filename
+        x = self.game_config['generated']['shield_wall_token']['min_x']
+        y = self.game_config['generated']['shield_wall_token']['min_y']
+        box_target = (
+            int(x),
+            int(y),
+            int(x+width_token),
+            int(y+height_token))
+        self.canvas.paste(token, box_target, mask=token)
+
     def render(self):
+        self.shieldWall()
         self.placeStorm()
         self.renderRegionMarks()
         self.renderFactionPositions()
@@ -435,17 +471,17 @@ class Renderer:
         del filename
         card = token
         # render card content
-        text = card_object['header']
+        text = card_object.get('header', '')
         x, y = 10, 10
         for line in textwrap.wrap(text, width=12):
             card, w, h = self.renderText(line, self.fnt_card_large, 'white', x, y, anchor=None, canvas=card)
             y += h + 9
         y += 10
-        text = card_object['description']
+        text = card_object.get('description', '')
         for line in textwrap.wrap(text, width=30):
             card, w, h = self.renderText(line, self.fnt_card_tiny, 'white', x, y, anchor=None, canvas=card)
             y += 14
-        text = card_object['type']
+        text = card_object.get('type', '')
         text_subtype = card_object.get('subtype', None)
         if text_subtype is not None:
             text += ' / ' + text_subtype
@@ -496,12 +532,12 @@ class Renderer:
             box_target = (x+half_width-dx, y+half_width-dy, x+half_width+dx, y+half_width+dy)
             self.canvas.paste(token, box_target, mask=token)
         y_txt = y + self.txt_spacing_wheel
-        if username_area is not None:
+        if username_area is not None and username_area in self.game_state['visual'].keys():
             text = self.game_state['visual'][username_area]
             self.canvas, w, h = self.renderText(text, self.fnt_wheel, 'black', x, y_txt, ycenter=True)
             y_txt += h
             y_txt += self.txt_spacing_wheel
-        if faction_area is not None:
+        if faction_area is not None and faction_area in self.game_state['visual'].keys():
             text = self.game_state['visual'][faction_area]
             self.canvas, w, h = self.renderText(text, self.fnt_wheel, 'black', x, y_txt, ycenter=True)
 
@@ -518,6 +554,12 @@ class Renderer:
         canvas = Image.alpha_composite(canvas, txt)
         return canvas, w, h
 
+    def extractCards(self, cards_object):
+        lst = []
+        for deck, cards in cards_object.items():
+            lst += cards
+        return lst
+
     def renderBattle(self):
         if not self.battle:
             return None
@@ -542,35 +584,37 @@ class Renderer:
         cards = self.game_state['areas'].get('wheel_attacker_cards', None)
         width = xthird
         if cards is not None:
-            for j, key in enumerate(cards):
+            cards_as_list = self.extractCards(cards)
+            for j, key in enumerate(cards_as_list):
                 card_object = self.card_objects[key]
                 card, w, h = self.render_card(card_object)
                 half_width = int(w/2)
                 half_height = int(h/2)
-                xoff = j*(w+self.card_spacing)-int(len(cards)*(w+self.card_spacing)/2)
+                xoff = j*(w+self.card_spacing)-int(len(cards_as_list)*(w+self.card_spacing)/2)
                 box_target = (x+xoff, y-half_height, x+xoff+w, y+half_height)
                 self.canvas.paste(card, box_target, mask=card)
             y += width + self.card_spacing
-        angle = self.game_state['visual']['wheel_attacker_value']
+        angle = self.game_state['visual'].get('wheel_attacker_value', 0)
         leader = self.game_state['visual'].get('wheel_attacker_leader', None)
         username_area = 'wheel_attacker_player_name'
         faction_area = 'wheel_attacker_player_faction'
         self.placeWheel(x, y, width, angle, username_area=username_area, faction_area=faction_area, leader=leader)
         # place right wheel
         x = self.width_canvas-int(xthird-xthird/5)
-        y = self.height_canvas-int(ythird-ythird/3)
+        y = self.height_canvas-int(ythird)
         cards = self.game_state['areas'].get('wheel_defender_cards', None)
         if cards is not None:
-            for j, key in enumerate(cards):
+            cards_as_list = self.extractCards(cards)
+            for j, key in enumerate(cards_as_list):
                 card_object = self.card_objects[key]
                 card, w, h = self.render_card(card_object)
                 half_width = int(w/2)
                 half_height = int(h/2)
-                xoff = j*(w+self.card_spacing)-int(len(cards)*(w+self.card_spacing)/2)
+                xoff = j*(w+self.card_spacing)-int(len(cards_as_list)*(w+self.card_spacing)/2)
                 box_target = (x+xoff, y-half_height, x+xoff+w, y+half_height)
                 self.canvas.paste(card, box_target, mask=card)
             y -= width + self.card_spacing
-        angle = self.game_state['visual']['wheel_defender_value']
+        angle = self.game_state['visual'].get('wheel_defender_value', 0)
         leader = self.game_state['visual'].get('wheel_defender_leader', None)
         username_area = 'wheel_defender_player_name'
         faction_area = 'wheel_defender_player_faction'
